==> contentsquare-1.1.0.tar.gz: test/src/modules/webview_tracking/webview_tracking_test.dart <==
import 'package:contentsquare/src/modules/webview_tracking/screengraph_retrieval.dart';
import 'package:contentsquare/src/modules/webview_tracking/logs/logs.dart';
import 'package:contentsquare/src/modules/webview_tracking/webview_tracking.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mocktail/mocktail.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../../../helpers/decompress_tvp.dart';
import '../../../helpers/expect_logs.dart';
import '../../../helpers/matchers/fixtures/matcher.dart';
import '../../../helpers/mocks/fake_webview.dart';

class _MockWebViewController extends Mock implements WebViewController {}

void main() {
  ContentsquareWebViewTrackerImpl buildDummyWebViewTrackerImplementation() {
    return ContentsquareWebViewTrackerImpl(
      screengraphRetrievalConfiguration: ScreengraphRetrievalConfiguration(
        maxExecutionDuration: const Duration(seconds: 10),
        intervalBetweenJSCalls: const Duration(milliseconds: 100),
      ),
      getEncodedWebViewTrackerTVP: () => null,
    );
  }

  group('$ContentsquareWebViewTrackerBuilder', () {
    testWidgets(
      'Should build a WebView tracked by the $ContentsquareWebViewTrackerBuilder',
      (WidgetTester tester) async {
        await tester.pumpWidget(
          MaterialApp(
            home: ContentsquareWebViewTrackerBuilder(
              builder: (context, tracker) {
                return WebView(
                  initialUrl:
                      'https://tupeuxpastest.csq.io/mobile/web-view-cart.html',
                  javascriptMode: JavascriptMode.unrestricted,
                  onWebViewCreated: tracker.onWebViewCreated,
                  onPageFinished: tracker.onPageFinished,
                  javascriptChannels: tracker.javascriptChannels,
                );
              },
            ),
          ),
        );

        final webview = tester.element(find.byType(WebView));

        expect(webview, isNotNull);
      },
    );

    group('getEncodedWebViewTrackerTVP', () {
      testWidgets(
        'Should return the TVP of the WebViewTracker',
        (WidgetTester tester) async {
          late ContentsquareWebViewTrackerImpl trackerImpl;

          await tester.pumpWidget(
            MaterialApp(
              home: ContentsquareWebViewTrackerBuilder(
                builder: (_, tracker) {
                  trackerImpl = (tracker as ContentsquareWebViewTrackerImpl);
                  return const WebView(
                    initialUrl: 'https://www.google.com',
                  );
                },
              ),
            ),
          );

          final tvp = trackerImpl.getEncodedWebViewTrackerTVP();

          expect(decompressTVP(tvp: tvp!), matchesFixture('webviewTVP'));
        },
      );

      testWidgets(
        'Should return null and log a $WebViewTrackerNotMountedLog if the WebViewTracker is not mounted',
        (WidgetTester tester) async {
          late ContentsquareWebViewTrackerImpl trackerImpl;

          await tester.pumpWidget(
            MaterialApp(
              home: Builder(
                builder: (context) {
                  return GestureDetector(
                    behavior: HitTestBehavior.opaque,
                    onTap: () {
                      // Replace the page with a new one to unmount the
                      // ContentsquareWebViewTrackerBuilder
                      Navigator.of(context).pushReplacement(
                        MaterialPageRoute(builder: (_) => const SizedBox()),
                      );
                    },
                    child: ContentsquareWebViewTrackerBuilder(
                      builder: (_, tracker) {
                        trackerImpl =
                            (tracker as ContentsquareWebViewTrackerImpl);
                        return const WebView(
                            initialUrl: 'https://www.google.com');
                      },
                    ),
                  );
                },
              ),
            ),
          );

          await tester.tap(find.byType(GestureDetector));
          await tester.pumpAndSettle();

          expectLogs((log) => expect(log, isA<WebViewTrackerNotMountedLog>()));

          final webviewTVP = trackerImpl.getEncodedWebViewTrackerTVP();

          expect(webviewTVP, isNull);
        },
      );

      testWidgets(
        'Should return null and log a $WebViewTrackerMissingRenderObjectLog if the flutter trees are only partially built',
        (WidgetTester tester) async {
          late final String? webviewTVP;

          expectLogs(
            // Use [expectSync] because this is triggered inside a [Future] (tester.pumpWidget)
            (log) => expectSync(
              log,
              isA<WebViewTrackerMissingRenderObjectLog>(),
            ),
          );

          await tester.pumpWidget(
            MaterialApp(
              home: ContentsquareWebViewTrackerBuilder(
                builder: (_, tracker) {
                  final trackerImpl =
                      (tracker as ContentsquareWebViewTrackerImpl);

                  // Trigger the error log by calling getEncodedWebViewTrackerTVP
                  // in the middle of the build
                  webviewTVP = trackerImpl.getEncodedWebViewTrackerTVP();
                  return const WebView(initialUrl: 'https://www.google.com');
                },
              ),
            ),
          );

          expect(webviewTVP, isNull);
        },
      );

      testWidgets(
        'Should return null and log a $WebViewTrackerMissingGlobalFrameLog if the widgets have been built but not laid out',
        (WidgetTester tester) async {
          late final String? webviewTVP;

          expectLogs(
            // Use [expectSync] because this is triggered inside a [Future] (tester.pumpWidget)
            (log) => expectSync(
              log,
              isA<WebViewTrackerMissingGlobalFrameLog>(),
            ),
          );

          await tester.pumpWidget(
            MaterialApp(
              home: ContentsquareWebViewTrackerBuilder(
                builder: (_, tracker) {
                  final trackerImpl =
                      (tracker as ContentsquareWebViewTrackerImpl);

                  // LayoutBuilder delays the build of the tree until the widget
                  // above have been laid out
                  return LayoutBuilder(builder: (context, _) {
                    // At this point, the [ContentsquareWebViewRenderObject]
                    // has been built (since it is a parent of the [LayoutBuilder])
                    // but not laid out (since the build phase is not over)
                    webviewTVP = trackerImpl.getEncodedWebViewTrackerTVP();
                    return const WebView(initialUrl: 'https://www.google.com');
                  });
                },
              ),
            ),
          );

          expect(webviewTVP, isNull);
        },
      );
    });
  });

  group(
    '$ContentsquareWebViewTrackerBuilder injection of the bridge after onWebViewCreated and onPageFinished have been called',
    () {
      test(
        'should log a a $WebViewInitializationErrorLog on any other platform than Android or IOS',
        () async {
          debugDefaultTargetPlatformOverride = TargetPlatform.fuchsia;

          final fakeWebViewController = FakeWebViewController(
            onRunJavascript: ((javascriptString) async {}),
            onRunJavascriptReturningResult: (_) {
              return Future.value('');
            },
          );

          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.onWebViewCreated(fakeWebViewController);

          expectLogs(
            (log) => expect(log, isA<WebViewInitializationErrorLog>()),
          );

          await tracker.onPageFinished('url');
        },
      );

      test(
        'should log a $WebViewInitializationErrorLog if a controller has not been bound to the builder and the web view has finished to load',
        () async {
          final tracker = buildDummyWebViewTrackerImplementation();

          expectLogs(
            (log) => expect(log, isA<WebViewInitializationErrorLog>()),
          );

          await tracker.onPageFinished('url');
        },
      );

      test(
        'should return normally on Android',
        () async {
          debugDefaultTargetPlatformOverride = TargetPlatform.android;
          const _androidJavascriptTrueString = 'true';

          final fakeWebViewController = FakeWebViewController(
            onRunJavascript: ((_) async {}),
            onRunJavascriptReturningResult: (_) {
              return Future.value(_androidJavascriptTrueString);
            },
          );

          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.onWebViewCreated(fakeWebViewController);

          await expectLater(
            () => tracker.onPageFinished('url'),
            returnsNormally,
          );
        },
      );

      test(
        'should return normally on iOS',
        () async {
          debugDefaultTargetPlatformOverride = TargetPlatform.iOS;
          const _iosJavascriptTrueString = '1';

          final fakeWebViewController = FakeWebViewController(
            onRunJavascript: ((_) async {}),
            onRunJavascriptReturningResult: (_) {
              return Future.value(_iosJavascriptTrueString);
            },
          );

          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.onWebViewCreated(fakeWebViewController);

          await expectLater(
            () => tracker.onPageFinished('url'),
            returnsNormally,
          );
        },
      );

      test(
        'The bridge implementation should match its fixture',
        () async {
          debugDefaultTargetPlatformOverride = TargetPlatform.iOS;
          const _iosJavascriptTrueString = '1';

          final mockWebViewController = _MockWebViewController();
          when(
            () => mockWebViewController.runJavascriptReturningResult(any()),
          ).thenAnswer((_) async => _iosJavascriptTrueString);
          when(
            () => mockWebViewController.currentUrl(),
          ).thenAnswer((_) async => 'url');

          late final String bridge;
          when(
            () => mockWebViewController.runJavascript(any()),
          ).thenAnswer((invocation) async {
            bridge = invocation.positionalArguments.first;
          });

          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.onWebViewCreated(mockWebViewController);
          await tracker.onPageFinished('url');

          await expectLater(
            bridge,
            matchesFixture(
              'expectedBridgeJSToDartToInject',
              isVersionSpecific: false,
            ),
          );
        },
      );
    },
  );
}

==> contentsquare-1.3.0.tar.gz: test/src/modules/webview_tracking/webview_tracking_test.dart <==
import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/modules/webview_tracking/logs/logs.dart';
import 'package:contentsquare/src/modules/webview_tracking/screengraph_retrieval.dart';
import 'package:contentsquare/src/modules/webview_tracking/webview_tracking.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../../../helpers/decompress_tvp.dart';
import '../../../helpers/expect_logs.dart';
import '../../../helpers/matchers/fixtures/matcher.dart';

void main() {
  ContentsquareWebViewTrackerImpl buildDummyWebViewTrackerImplementation() {
    return ContentsquareWebViewTrackerImpl(
      screengraphRetrievalConfiguration: ScreengraphRetrievalConfiguration(
        maxExecutionDuration: const Duration(seconds: 10),
        intervalBetweenJSCalls: const Duration(milliseconds: 100),
      ),
      getEncodedWebViewTrackerTVP: () => null,
    );
  }

  group('$ContentsquareWebViewTrackerBuilder', () {
    testWidgets(
      'Should build a WebView tracked by the $ContentsquareWebViewTrackerBuilder',
      (WidgetTester tester) async {
        await tester.pumpWidget(
          MaterialApp(
            home: ContentsquareWebViewTrackerBuilder(
              builder: (context, tracker) {
                return WebView(
                  initialUrl:
                      'https://tupeuxpastest.csq.io/mobile/web-view-cart.html',
                  javascriptMode: JavascriptMode.unrestricted,
                  onWebViewCreated: tracker.onWebViewCreated,
                  onPageFinished: tracker.onPageFinished,
                  javascriptChannels: tracker.javascriptChannels,
                );
              },
            ),
          ),
        );

        final webview = tester.element(find.byType(WebView));

        expect(webview, isNotNull);
      },
    );

    group('getEncodedWebViewTrackerTVP', () {
      testWidgets(
        'Should return the TVP of the WebViewTracker',
        (WidgetTester tester) async {
          late ContentsquareWebViewTrackerImpl trackerImpl;

          await tester.pumpWidget(
            MaterialApp(
              home: ContentsquareWebViewTrackerBuilder.custom(
                builder: (_, tracker) {
                  trackerImpl = (tracker as ContentsquareWebViewTrackerImpl);
                  return const WebView(
                    initialUrl: 'https://www.google.com',
                  );
                },
              ),
            ),
          );

          final tvp = trackerImpl.getEncodedWebViewTrackerTVP();

          expect(decompressTVP(tvp: tvp!), matchesFixture('webviewTVP'));
        },
      );

      testWidgets(
        'Should return null and log a $WebViewTrackerNotMountedLog if the WebViewTracker is not mounted',
        (WidgetTester tester) async {
          late ContentsquareWebViewTrackerImpl trackerImpl;

          await tester.pumpWidget(
            MaterialApp(
              home: Builder(
                builder: (context) {
                  return GestureDetector(
                    behavior: HitTestBehavior.opaque,
                    onTap: () {
                      // Replace the page with a new one to unmount the
                      // ContentsquareWebViewTrackerBuilder
                      Navigator.of(context).pushReplacement(
                        MaterialPageRoute(builder: (_) => const SizedBox()),
                      );
                    },
                    child: ContentsquareWebViewTrackerBuilder.custom(
                      builder: (_, tracker) {
                        trackerImpl =
                            (tracker as ContentsquareWebViewTrackerImpl);
                        return const WebView(
                            initialUrl: 'https://www.google.com');
                      },
                    ),
                  );
                },
              ),
            ),
          );

          await tester.tap(find.byType(GestureDetector));
          await tester.pumpAndSettle();

          expectLogs((log) => expect(log, isA<WebViewTrackerNotMountedLog>()));

          final webviewTVP = trackerImpl.getEncodedWebViewTrackerTVP();

          expect(webviewTVP, isNull);
        },
      );

      testWidgets(
        'Should return null and log a $WebViewTrackerMissingRenderObjectLog if the flutter trees are only partially built',
        (WidgetTester tester) async {
          late final String? webviewTVP;

          expectLogs(
            // Use [expectSync] because this is triggered inside a [Future] (tester.pumpWidget)
            (log) => expectSync(
              log,
              isA<WebViewTrackerMissingRenderObjectLog>(),
            ),
          );

          await tester.pumpWidget(
            MaterialApp(
              home: ContentsquareWebViewTrackerBuilder.custom(
                builder: (_, tracker) {
                  final trackerImpl =
                      (tracker as ContentsquareWebViewTrackerImpl);

                  // Trigger the error log by calling getEncodedWebViewTrackerTVP
                  // in the middle of the build
                  webviewTVP = trackerImpl.getEncodedWebViewTrackerTVP();
                  return const WebView(initialUrl: 'https://www.google.com');
                },
              ),
            ),
          );

          expect(webviewTVP, isNull);
        },
      );

      testWidgets(
        'Should return null and log a $WebViewTrackerMissingGlobalFrameLog if the widgets have been built but not laid out',
        (WidgetTester tester) async {
          late final String? webviewTVP;

          expectLogs(
            // Use [expectSync] because this is triggered inside a [Future] (tester.pumpWidget)
            (log) => expectSync(
              log,
              isA<WebViewTrackerMissingGlobalFrameLog>(),
            ),
          );

          await tester.pumpWidget(
            MaterialApp(
              home: ContentsquareWebViewTrackerBuilder.custom(
                builder: (_, tracker) {
                  final trackerImpl =
                      (tracker as ContentsquareWebViewTrackerImpl);

                  // LayoutBuilder delays the build of the tree until the widget
                  // above have been laid out
                  return LayoutBuilder(builder: (context, _) {
                    // At this point, the [ContentsquareWebViewRenderObject]
                    // has been built (since it is a parent of the [LayoutBuilder])
                    // but not laid out (since the build phase is not over)
                    webviewTVP = trackerImpl.getEncodedWebViewTrackerTVP();
                    return const WebView(initialUrl: 'https://www.google.com');
                  });
                },
              ),
            ),
          );

          expect(webviewTVP, isNull);
        },
      );
    });
  });

  group(
    '$ContentsquareWebViewTrackerBuilder injection of the bridge after onWebViewCreated and onPageFinished have been called',
    () {
      test(
        'should log a $WebViewInitializationErrorLog if a controller has not been registered',
        () async {
          final tracker = buildDummyWebViewTrackerImplementation();

          expectLogs(
            (log) => expect(log, isA<WebViewInitializationErrorLog>()),
          );

          await tracker.initializeWebViewTracking();
        },
      );

      test(
        'should log a $WebViewInitializationErrorLog if isChannelAvailable returns false when initializeWebViewTracking is called',
        () async {
          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.registerWebViewController(
            ContentsquareWebViewController(
              evaluateJavascript: (_) => throw UnimplementedError(),
              javascriptChannelCall: (_, __) => throw UnimplementedError(),
              isChannelAvailable: (_) async => false,
            ),
          );

          expectLogs(
            (log) => expect(log, isA<WebViewInitializationErrorLog>()),
          );

          await tracker.initializeWebViewTracking();
        },
      );

      test(
        'should return normally if the webview controller is registered and isChannelAvailable returns true when initializeWebViewTracking is called',
        () async {
          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.registerWebViewController(
            ContentsquareWebViewController(
              evaluateJavascript: (_) async => '',
              javascriptChannelCall: (_, __) => '',
              isChannelAvailable: (_) async => true,
            ),
          );

          expectLater(tracker.initializeWebViewTracking, returnsNormally);
        },
      );

      test(
        'The bridge implementation should match its fixture',
        () async {
          late String bridge;
          final tracker = buildDummyWebViewTrackerImplementation();

          tracker.registerWebViewController(
            ContentsquareWebViewController(
              evaluateJavascript: (value) async {
                bridge = value;
                return '';
              },
              javascriptChannelCall: (channelName, data) {
                return '$channelName($data)';
              },
            ),
          );

          expectLater(tracker.initializeWebViewTracking, returnsNormally);

          await expectLater(
            bridge,
            matchesFixture(
              'expectedBridgeJSToDartToInject',
              isVersionSpecific: false,
            ),
          );
        },
      );
    },
  );
}
